#!/usr/bin/env python3
"""
This file generates word tables from the profiling data in the results folders.
It uses the 'latextable' package to create the Latex syntax tables, and 'pandas'
for the data mangement:
 - pip install --user te latextable pandas
Don't forget to load the cray-python module before running the script!
"""

import latextable as lt
import pandas as pd
from texttable import Texttable
from typing import List


def get_profile_data(nproma: int, itteration: int) -> pd.DataFrame:
    """Load the profiling data from the results and aggregate hardware counters."""
    df = pd.read_csv(f"../profiling/res-gpu_prof_nproma_{nproma}/rocprof.csv")

    # Get starting indexes of each iteration, and filter out rows of 7th iteration.
    start_kernel = "eltinv$eltinv_mod_$ck_L136_2_cce$noloop$form.kd"
    start_idxs = df.loc[df["KernelName"] == start_kernel].index
    itter_df = df.iloc[start_idxs[itteration - 1]:start_idxs[itteration]]
    
    return itter_df


def generate_metrics_table(kernelName: str, npromas: List[int]):
    """Adds new section with table containing transfer data."""
    # Load the data from the 7th iteration for each nproma value.
    df_nproma32 = get_profile_data(32, 7)
    # df_nproma64 = get_profile_data(64, 7)
    # df_nproma128 = get_profile_data(128, 7)
    # df_nproma192 = get_profile_data(192, 7)
    # df_nproma256 = get_profile_data(256, 7)
    
    # Extract the kernel to create the table for.
    df_nproma32 = df_nproma32.loc[df_nproma32["KernelName"] == kernelName]
    # df_nproma64 = df_nproma64.loc[df_nproma64["KernelName"] == kernelName]
    # df_nproma128 = df_nproma128.loc[df_nproma128["KernelName"] == kernelName]
    # df_nproma192 = df_nproma192.loc[df_nproma192["KernelName"] == kernelName]
    # df_nproma256 = df_nproma256.loc[df_nproma256["KernelName"] == kernelName]

    # Setup table with a column per nproma and row per metric.
    table = Texttable()
    align = ["c" for _ in npromas]
    align.insert(0, "l")
    table.set_cols_align(align)
    # table.set_cols_valign(["m", "m", "b"])
    
    # Setup table header and fill in table.
    header = [f"nproma={nprom}" for nprom in npromas]
    header.insert(0, "Metric")
    rows = [header]
    
    # Add values for each metric selected.
    metrics = ["vgpr", "sgpr", "Wavefronts", "grd", "wgr", "VALUUtilization", 
               "VALUBusy", "MemUnitBusy", "L2CacheHit"]
    
    for metric in metrics:
        rows.append([metric, df_nproma32.iloc[0][metric]])
        print(rows)
        exit(1)
    

if __name__ == "__main__":
    # Create a table for each of the kernels in the list.
    kernels = [
        "easre1b$easre1b_mod_$ck_L75_1.kd",
        "efourier_in$efourier_in_mod_$ck_L61_2_cce$noloop$form.kd",
        "trltog_cudocdaaware$trltog_mod_$ck_L559_4.kd",
        "trgtol_cudaaware$trgtol_mod_$ck_L563_3.kd",
        "efourier_out$efourier_out_mod_$ck_L56_1.kd",
        "eprfi2b$eprfi2b_mod_$ck_L84_1_cce$noloop$form.kd",
    ]
    npromas = [32, 64, 128, 192, 256]
    
    for kernel in kernels:
        generate_metrics_table(kernel, npromas)
        exit(1)
    
    # Add metrics table.
    # add_metrics_table(doc, "easre1b$easre1b_mod_$ck_L75_1")
    # add_metrics_table(doc, "efourier_in$efourier_in_mod_$ck_L61_2_cce$noloop$form")
    # add_metrics_table(doc, "trltog_cudocdaaware$trltog_mod_$ck_L559_4")
    # add_metrics_table(doc, "trgtol_cudaaware$trgtol_mod_$ck_L563_3")
    # add_metrics_table(doc, "efourier_out$efourier_out_mod_$ck_L56_1")
    # add_metrics_table(doc, "eprfi2b$eprfi2b_mod_$ck_L84_1_cce$noloop$form")    
    
    # Save the document in the tables folder.
    # doc.save("tables/ecTrans_dwarf_tables.docx")
